@@ -1,8 +1,8 @@
 // qsamplerChannelStrip.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, 2008 Christian Schoenebeck
+   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007, 2008, 2014 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,10 +29,14 @@
 
 #include <QMessageBox>
 #include <QDragEnterEvent>
+#include <QFileInfo>
+#include <QTimer>
 #include <QUrl>
+#include <QMenu>
 
-#define MIDI_OFF_COLOR			Qt::darkGreen
-#define MIDI_ON_COLOR			Qt::green
+#if QT_VERSION >= 0x050000
+#include <QMimeData>
+#endif
 
 // Channel status/usage usage limit control.
 #define QSAMPLER_ERROR_LIMIT	3
@@ -57,6 +61,11 @@ namespace QSampler {
 // QSampler::ChannelStrip -- Channel strip form implementation.
 //
 
+// MIDI activity pixmap common resources.
+int      ChannelStrip::g_iMidiActivityRefCount = 0;
+QPixmap *ChannelStrip::g_pMidiActivityLedOn    = NULL;
+QPixmap *ChannelStrip::g_pMidiActivityLedOff   = NULL;
+
 // Channel strip activation/selection.
 ChannelStrip *ChannelStrip::g_pSelectedStrip = NULL;
 
@@ -69,6 +78,26 @@ ChannelStrip::ChannelStrip ( QWidget* pParent, Qt::WindowFlags wflags )
 	m_pChannel     = NULL;
 	m_iDirtyChange = 0;
 	m_iErrorCount  = 0;
+	m_instrumentListPopupMenu = NULL;
+
+	if (++g_iMidiActivityRefCount == 1) {
+		g_pMidiActivityLedOn  = new QPixmap(":/images/ledon1.png");
+		g_pMidiActivityLedOff = new QPixmap(":/images/ledoff1.png");
+	}
+
+	m_ui.MidiActivityLabel->setPixmap(*g_pMidiActivityLedOff);
+
+#ifndef CONFIG_EVENT_CHANNEL_MIDI
+	m_ui.MidiActivityLabel->setToolTip("MIDI activity (disabled)");
+#endif
+
+	m_pMidiActivityTimer = new QTimer(this);
+	m_pMidiActivityTimer->setSingleShot(true);
+
+	QObject::connect(m_pMidiActivityTimer,
+		SIGNAL(timeout()),
+		SLOT(midiActivityLedOff())
+	);
 
 	// Try to restore normal window positioning.
 	adjustSize();
@@ -95,21 +124,6 @@ ChannelStrip::ChannelStrip ( QWidget* pParent, Qt::WindowFlags wflags )
 		SIGNAL(clicked()),
 		SLOT(channelFxEdit()));
 
-	pMidiActivityTimer = new QTimer(this);
-	pMidiActivityTimer->setSingleShot(true);
-	QObject::connect(
-		pMidiActivityTimer, SIGNAL(timeout()),
-		this, SLOT(midiDataCeased())
-	);
-
-#if CONFIG_EVENT_CHANNEL_MIDI
-	m_ui.MidiActivityLabel->setPalette(MIDI_OFF_COLOR);
-	m_ui.MidiActivityLabel->setAutoFillBackground(true);
-#else
-	m_ui.MidiActivityLabel->setText("X");
-	m_ui.MidiActivityLabel->setToolTip("MIDI Activity Disabled");
-#endif
-
 	setSelected(false);
 }
 
@@ -122,6 +136,15 @@ ChannelStrip::~ChannelStrip (void)
 	if (m_pChannel)
 		delete m_pChannel;
 	m_pChannel = NULL;
+
+	if (--g_iMidiActivityRefCount == 0) {
+		if (g_pMidiActivityLedOn)
+			delete g_pMidiActivityLedOn;
+		g_pMidiActivityLedOn = NULL;
+		if (g_pMidiActivityLedOff)
+			delete g_pMidiActivityLedOff;
+		g_pMidiActivityLedOff = NULL;
+	}
 }
 
 
@@ -140,7 +163,8 @@ void ChannelStrip::dragEnterEvent ( QDragEnterEvent* pDragEnterEvent )
 			while (iter.hasNext()) {
 				const QString& sFilename = iter.next().toLocalFile();
 				if (!sFilename.isEmpty()) {
-					bAccept = Channel::isInstrumentFile(sFilename);
+				//	bAccept = Channel::isDlsInstrumentFile(sFilename);
+					bAccept = QFileInfo(sFilename).exists();
 					break;
 				}
 			}
@@ -217,7 +241,7 @@ void ChannelStrip::setDisplayFont ( const QFont & font )
 {
 	m_ui.EngineNameTextLabel->setFont(font);
 	m_ui.MidiPortChannelTextLabel->setFont(font);
-	m_ui.InstrumentNameTextLabel->setFont(font);
+	m_ui.InstrumentNamePushButton->setFont(font);
 	m_ui.InstrumentStatusTextLabel->setFont(font);
 }
 
@@ -227,17 +251,19 @@ void ChannelStrip::setDisplayEffect ( bool bDisplayEffect )
 {
 	QPalette pal;
 	pal.setColor(QPalette::Foreground, Qt::yellow);
+	pal.setColor(QPalette::ButtonText, Qt::yellow);
 	m_ui.EngineNameTextLabel->setPalette(pal);
 	m_ui.MidiPortChannelTextLabel->setPalette(pal);
 	pal.setColor(QPalette::Foreground, Qt::green);
+	pal.setColor(QPalette::ButtonText, Qt::green);
 	if (bDisplayEffect) {
-		QPixmap pm(":/icons/displaybg1.png");
+		QPixmap pm(":/images/displaybg1.png");
 		pal.setBrush(QPalette::Background, QBrush(pm));
 	} else {
 		pal.setColor(QPalette::Background, Qt::black);
 	}
 	m_ui.ChannelInfoFrame->setPalette(pal);
-	m_ui.InstrumentNameTextLabel->setPalette(pal);
+	m_ui.InstrumentNamePushButton->setPalette(pal);
 	m_ui.StreamVoiceCountTextLabel->setPalette(pal);
 }
 
@@ -366,20 +392,67 @@ bool ChannelStrip::updateInstrumentName ( bool bForce )
 	// Instrument name...
 	if (m_pChannel->instrumentName().isEmpty()) {
 		if (m_pChannel->instrumentStatus() >= 0) {
-			m_ui.InstrumentNameTextLabel->setText(
+			m_ui.InstrumentNamePushButton->setText(
 				' ' + Channel::loadingInstrument());
 		} else {
-			m_ui.InstrumentNameTextLabel->setText(
+			m_ui.InstrumentNamePushButton->setText(
 				' ' + Channel::noInstrumentName());
 		}
 	} else {
-		m_ui.InstrumentNameTextLabel->setText(
+		m_ui.InstrumentNamePushButton->setText(
 			' ' + m_pChannel->instrumentName());
 	}
+	
+	bool bShowInstrumentPopup = false;
+
+	// Instrument list popup (for fast switching among sounds of the same file)
+	if (!m_pChannel->instrumentFile().isEmpty()) {
+		const QStringList instruments
+			= Channel::getInstrumentList(m_pChannel->instrumentFile(), true);
+		if (!instruments.isEmpty()) {
+			bShowInstrumentPopup = true;
+			if (!m_instrumentListPopupMenu) {
+				m_instrumentListPopupMenu
+					= new QMenu(m_ui.InstrumentNamePushButton);
+				m_instrumentListPopupMenu->setTitle(tr("Instruments"));
+				// for cosmetical reasons, should have at least
+				// the width of the instrument name label...
+				m_instrumentListPopupMenu->setMinimumWidth(120);
+				m_ui.InstrumentNamePushButton->setMenu(m_instrumentListPopupMenu);
+				QObject::connect(m_instrumentListPopupMenu,
+					SIGNAL(triggered(QAction*)),
+					SLOT(instrumentListPopupItemClicked(QAction *)));
+			} else {
+				m_instrumentListPopupMenu->clear();
+			}
+			QAction *action;
+			for (int i = 0; i < instruments.size(); ++i) {
+				action = m_instrumentListPopupMenu->addAction(instruments.at(i));
+				action->setData(i);
+				action->setCheckable(true);
+				action->setChecked(i == m_pChannel->instrumentNr());
+			}
+		}
+	}
+
+	if (!bShowInstrumentPopup && m_instrumentListPopupMenu) {
+		delete m_instrumentListPopupMenu;
+		m_instrumentListPopupMenu = NULL;
+	}
 
 	return true;
 }
 
+void ChannelStrip::instrumentListPopupItemClicked ( QAction *action )
+{
+	if (!action) return;
+
+	QVariant data = action->data();
+	if (data.isValid() && !m_pChannel->instrumentFile().isEmpty()) {
+		m_pChannel->loadInstrument(m_pChannel->instrumentFile(), data.toInt());
+		emit channelChanged(this);
+	}
+}
 
 // Do the dirty volume change.
 bool ChannelStrip::updateChannelVolume (void)
@@ -451,7 +524,7 @@ bool ChannelStrip::updateChannelInfo (void)
 	const QColor& rgbFore = pal.color(QPalette::Foreground);
 
 	// Instrument status...
-	int iInstrumentStatus = m_pChannel->instrumentStatus();
+	const int iInstrumentStatus = m_pChannel->instrumentStatus();
 	if (iInstrumentStatus < 0) {
 		pal.setColor(QPalette::Foreground, Qt::red);
 		m_ui.InstrumentStatusTextLabel->setPalette(pal);
@@ -460,6 +533,7 @@ bool ChannelStrip::updateChannelInfo (void)
 		m_iErrorCount++;
 		return false;
 	}
+
 	// All seems normal...
 	pal.setColor(QPalette::Foreground,
 		iInstrumentStatus < 100 ? Qt::yellow : Qt::green);
@@ -472,12 +546,15 @@ bool ChannelStrip::updateChannelInfo (void)
 	// Mute/Solo button state coloring...
 	bool bMute = m_pChannel->channelMute();
 	const QColor& rgbButton = pal.color(QPalette::Button);
+	const QColor& rgbButtonText = pal.color(QPalette::ButtonText);
 	pal.setColor(QPalette::Foreground, rgbFore);
 	pal.setColor(QPalette::Button, bMute ? Qt::yellow : rgbButton);
+	pal.setColor(QPalette::ButtonText, bMute ? Qt::darkYellow : rgbButtonText);
 	m_ui.ChannelMutePushButton->setPalette(pal);
 	m_ui.ChannelMutePushButton->setDown(bMute);
 	bool bSolo = m_pChannel->channelSolo();
 	pal.setColor(QPalette::Button, bSolo ? Qt::cyan : rgbButton);	
+	pal.setColor(QPalette::ButtonText, bSolo ? Qt::darkCyan : rgbButtonText);
 	m_ui.ChannelSoloPushButton->setPalette(pal);
 	m_ui.ChannelSoloPushButton->setDown(bSolo);
 #else
@@ -549,10 +626,6 @@ void ChannelStrip::volumeChanged ( int iVolume )
 	}
 }
 
-void ChannelStrip::midiArrived() {
-	m_ui.MidiActivityLabel->setPalette(MIDI_ON_COLOR);
-	pMidiActivityTimer->start(50);
-}
 
 // Context menu event handler.
 void ChannelStrip::contextMenuEvent( QContextMenuEvent *pEvent )
@@ -564,10 +637,20 @@ void ChannelStrip::contextMenuEvent( QContextMenuEvent *pEvent )
 	m_pChannel->contextMenuEvent(pEvent);
 }
 
-void ChannelStrip::midiDataCeased() {
-	m_ui.MidiActivityLabel->setPalette(MIDI_OFF_COLOR);
+
+void ChannelStrip::midiActivityLedOn (void)
+{
+	m_ui.MidiActivityLabel->setPixmap(*g_pMidiActivityLedOn);
+	m_pMidiActivityTimer->start(100);
 }
 
+
+void ChannelStrip::midiActivityLedOff (void)
+{
+	m_ui.MidiActivityLabel->setPixmap(*g_pMidiActivityLedOff);
+}
+
+
 // Error count hackish accessors.
 void ChannelStrip::resetErrorCount (void)
 {
