@@ -1,7 +1,7 @@
 // qsamplerOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -109,6 +109,15 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	QObject::connect(m_ui.ConfirmRemoveCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.ConfirmResetCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.ConfirmRestartCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.ConfirmErrorCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(optionsChanged()));
 	QObject::connect(m_ui.KeepOnTopCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(optionsChanged()));
@@ -133,11 +142,11 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	QObject::connect(m_ui.MaxStreamsSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(maxStreamsChanged(int)));
-	QObject::connect(m_ui.OkPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(accepted()),
 		SLOT(accept()));
-	QObject::connect(m_ui.CancelPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(rejected()),
 		SLOT(reject()));
 }
 
@@ -212,6 +221,9 @@ void OptionsForm::setup ( Options *pOptions )
 
 	// Other options finally.
 	m_ui.ConfirmRemoveCheckBox->setChecked(m_pOptions->bConfirmRemove);
+	m_ui.ConfirmRestartCheckBox->setChecked(m_pOptions->bConfirmRestart);
+	m_ui.ConfirmResetCheckBox->setChecked(m_pOptions->bConfirmReset);
+	m_ui.ConfirmErrorCheckBox->setChecked(m_pOptions->bConfirmError);
 	m_ui.KeepOnTopCheckBox->setChecked(m_pOptions->bKeepOnTop);
 	m_ui.StdoutCaptureCheckBox->setChecked(m_pOptions->bStdoutCapture);
 	m_ui.CompletePathCheckBox->setChecked(m_pOptions->bCompletePath);
@@ -302,6 +314,9 @@ void OptionsForm::accept (void)
 		m_pOptions->iMessagesLimitLines = m_ui.MessagesLimitLinesSpinBox->value();
 		// Other options...
 		m_pOptions->bConfirmRemove = m_ui.ConfirmRemoveCheckBox->isChecked();
+		m_pOptions->bConfirmRestart = m_ui.ConfirmRestartCheckBox->isChecked();
+		m_pOptions->bConfirmReset  = m_ui.ConfirmResetCheckBox->isChecked();
+		m_pOptions->bConfirmError  = m_ui.ConfirmErrorCheckBox->isChecked();
 		m_pOptions->bKeepOnTop     = m_ui.KeepOnTopCheckBox->isChecked();
 		m_pOptions->bStdoutCapture = m_ui.StdoutCaptureCheckBox->isChecked();
 		m_pOptions->bCompletePath  = m_ui.CompletePathCheckBox->isChecked();
@@ -325,6 +340,9 @@ void OptionsForm::accept (void)
 	m_pOptions->saveComboBoxHistory(m_ui.ServerCmdLineComboBox);
 	m_pOptions->saveComboBoxHistory(m_ui.MessagesLogPathComboBox);
 
+	// Save/commit to disk.
+	m_pOptions->saveOptions();
+
 	// Just go with dialog acceptance.
 	QDialog::accept();
 }
@@ -394,7 +412,7 @@ void OptionsForm::stabilizeForm (void)
 	m_ui.MessagesLimitLinesSpinBox->setEnabled(
 		m_ui.MessagesLimitCheckBox->isChecked());
 
-	m_ui.OkPushButton->setEnabled(bValid);
+	m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->setEnabled(bValid);
 }
 
 
@@ -452,7 +470,7 @@ void OptionsForm::toggleDisplayEffect ( bool bOn )
 	QPalette pal;
 	pal.setColor(QPalette::Foreground, Qt::green);
 	if (bOn) {
-		QPixmap pm(":/icons/displaybg1.png");
+		QPixmap pm(":/images/displaybg1.png");
 		pal.setBrush(QPalette::Background, QBrush(pm));
 	} else {
 		pal.setColor(QPalette::Background, Qt::black);
